@@ -12,8 +12,8 @@
 //===----------------------------------------------------------------------===//
 //
 #include "polly/LinkAllPasses.h"
-#include "polly/Options.h"
 #include "polly/CodeGen/BlockGenerators.h"
+#include "polly/Options.h"
 #include "polly/ScopDetection.h"
 #include "polly/Support/ScopHelper.h"
 #include "llvm/Analysis/DominanceFrontier.h"
@@ -21,10 +21,10 @@
 #include "llvm/Analysis/PostDominators.h"
 #include "llvm/Analysis/RegionInfo.h"
 #include "llvm/Analysis/ValueTracking.h"
-#include "llvm/Transforms/Utils/Local.h"
+#include "llvm/IR/IntrinsicInst.h"
 #include "llvm/Support/CommandLine.h"
 #include "llvm/Support/Debug.h"
-
+#include "llvm/Transforms/Utils/Local.h"
 #include <vector>
 
 using namespace polly;
@@ -35,7 +35,7 @@ using namespace llvm;
 static cl::opt<bool> DisableIntraScopScalarToArray(
     "disable-polly-intra-scop-scalar-to-array",
     cl::desc("Do not rewrite scalar to array to generate independent blocks"),
-    cl::Hidden, cl::init(false), cl::cat(PollyCategory));
+    cl::Hidden, cl::init(true), cl::cat(PollyCategory));
 
 namespace {
 struct IndependentBlocks : public FunctionPass {
@@ -255,7 +255,8 @@ bool IndependentBlocks::createIndependentBlocks(BasicBlock *BB,
   Instruction *InsertPos = BB->getFirstNonPHIOrDbg();
 
   for (Instruction *Inst : WorkList)
-    moveOperandTree(Inst, R, ReplacedMap, InsertPos);
+    if (!isa<PHINode>(Inst))
+      moveOperandTree(Inst, R, ReplacedMap, InsertPos);
 
   // The BB was changed if we replaced any operand.
   return !ReplacedMap.empty();
@@ -276,7 +277,7 @@ bool IndependentBlocks::eliminateDeadCode(const Region *R) {
   // Find all trivially dead instructions.
   for (BasicBlock *BB : R->blocks())
     for (Instruction &Inst : *BB)
-      if (isInstructionTriviallyDead(&Inst))
+      if (!isIgnoredIntrinsic(&Inst) && isInstructionTriviallyDead(&Inst))
         WorkList.push_back(&Inst);
 
   if (WorkList.empty())
@@ -368,6 +369,8 @@ bool IndependentBlocks::translateScalarToArray(Instruction *Inst,
                                                const Region *R) {
   if (canSynthesize(Inst, LI, SE, R) && onlyUsedInRegion(Inst, R))
     return false;
+  if (isIgnoredIntrinsic(Inst))
+    return false;
 
   SmallVector<Instruction *, 4> LoadInside, LoadOutside;
   for (User *U : Inst->users())
@@ -449,6 +452,8 @@ bool IndependentBlocks::isIndependentBlock(const Region *R,
   for (Instruction &Inst : *BB) {
     if (canSynthesize(&Inst, LI, SE, R))
       continue;
+    if (isIgnoredIntrinsic(&Inst))
+      continue;
 
     // A value inside the Scop is referenced outside.
     for (User *U : Inst.users()) {
@@ -465,6 +470,8 @@ bool IndependentBlocks::isIndependentBlock(const Region *R,
       continue;
 
     for (Value *Op : Inst.operands()) {
+      if (isIgnoredIntrinsic(Op))
+        continue;
       if (isEscapeOperand(Op, BB, R)) {
         DEBUG(dbgs() << "Instruction in function '";
               BB->getParent()->printAsOperand(dbgs(), false);
@@ -498,8 +505,8 @@ void IndependentBlocks::getAnalysisUsage(AnalysisUsage &AU) const {
   AU.addPreserved<PostDominatorTree>();
   AU.addRequired<RegionInfoPass>();
   AU.addPreserved<RegionInfoPass>();
-  AU.addRequired<LoopInfo>();
-  AU.addPreserved<LoopInfo>();
+  AU.addRequired<LoopInfoWrapperPass>();
+  AU.addPreserved<LoopInfoWrapperPass>();
   AU.addRequired<ScalarEvolution>();
   AU.addPreserved<ScalarEvolution>();
   AU.addRequired<ScopDetection>();
@@ -507,10 +514,11 @@ void IndependentBlocks::getAnalysisUsage(AnalysisUsage &AU) const {
 }
 
 bool IndependentBlocks::runOnFunction(llvm::Function &F) {
+
   bool Changed = false;
 
   RI = &getAnalysis<RegionInfoPass>().getRegionInfo();
-  LI = &getAnalysis<LoopInfo>();
+  LI = &getAnalysis<LoopInfoWrapperPass>().getLoopInfo();
   SD = &getAnalysis<ScopDetection>();
   SE = &getAnalysis<ScalarEvolution>();
 
@@ -521,28 +529,14 @@ bool IndependentBlocks::runOnFunction(llvm::Function &F) {
   for (const Region *R : *SD) {
     Changed |= createIndependentBlocks(R);
     Changed |= eliminateDeadCode(R);
-    // This may change the RegionTree.
-    Changed |= splitExitBlock(const_cast<Region *>(R));
   }
 
-  DEBUG(dbgs() << "Before Scalar to Array------->\n");
-  DEBUG(F.dump());
-
-  for (const Region *R : *SD)
-    Changed |= translateScalarToArray(R);
-
-  DEBUG(dbgs() << "After Independent Blocks------------->\n");
-  DEBUG(F.dump());
-
   verifyAnalysis();
 
   return Changed;
 }
 
-void IndependentBlocks::verifyAnalysis() const {
-  for (const Region *R : *SD)
-    verifyScop(R);
-}
+void IndependentBlocks::verifyAnalysis() const {}
 
 void IndependentBlocks::verifyScop(const Region *R) const {
   assert(areAllBlocksIndependent(R) && "Cannot generate independent blocks");
@@ -555,7 +549,7 @@ Pass *polly::createIndependentBlocksPass() { return new IndependentBlocks(); }
 
 INITIALIZE_PASS_BEGIN(IndependentBlocks, "polly-independent",
                       "Polly - Create independent blocks", false, false);
-INITIALIZE_PASS_DEPENDENCY(LoopInfo);
+INITIALIZE_PASS_DEPENDENCY(LoopInfoWrapperPass);
 INITIALIZE_PASS_DEPENDENCY(RegionInfoPass);
 INITIALIZE_PASS_DEPENDENCY(ScalarEvolution);
 INITIALIZE_PASS_DEPENDENCY(ScopDetection);
